/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.service;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;

import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.richfaces.realworld.domain.Album;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.domain.User;

@Name("albumAction")
@Stateless
@AutoCreate
public class AlbumAction implements IAlbumAction {

	@In(value="entityManager")
	EntityManager em;
	
	@In FlushStrategy flushStrategy;

	public void addAlbum(Album album) {
		em.persist(album);
		album.getShelf().addAlbum(album);
		if(flushStrategy.isDatabaseStoreStrategy()){
			em.flush();
		}
	}
	
	public void deleteAlbum(Album album){
		if(album.getShelf()!=null){
			album.getShelf().setUnvisitedImages(new ArrayList<Image>());
		}
		album.getShelf().removeChildAlbum(album);
		em.remove(album);
		if(flushStrategy.isDatabaseStoreStrategy()){
			em.flush();
		}
	}
	
	public void editAlbum(Album album){
		if(flushStrategy.isDatabaseStoreStrategy()){
			em.flush();
		}
	}

	public List<Image> getUnvisitedImages(Album album){
		//TODO nick - use Shelf/Album.unvisitedImages
		return (List<Image>)em.createQuery("from Image i where i.album=:album and i.created >= :date").setParameter("album", album).setParameter("date", getDate()).getResultList();
	}
	
	private Date getDate() {
		//TODO nick - use Calendar.getInstance()
		Calendar c = Calendar.getInstance();
		c.add(Calendar.DAY_OF_YEAR, -15); 
		//TODO nick - return c.getTime()
		return c.getTime();
	}
}
